﻿using Blazor_Firestorer.Shared.Models;
using Google.Cloud.Firestore;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Blazor_Firestorer.Server.Data
{
    public class AlunoDAL : IAlunoDAL
    {
        string projectId;
        FirestoreDb fireStoreDb;

        public AlunoDAL()
        {
            string filepath = @"D:\_blazor\Blazor_Firestorer\Blazor_Firestorer\Server\FirestoreApiKey\blazorcrudfirestore-1565827c5156.json";
            Environment.SetEnvironmentVariable("GOOGLE_APPLICATION_CREDENTIALS", filepath);
            projectId = "blazorcrudfirestore";
            fireStoreDb = FirestoreDb.Create(projectId);
        }

        public async Task<List<Aluno>> GetAlunos()
        {
            try
            {
                Query alunoQuery = fireStoreDb.Collection("alunos");
                QuerySnapshot alunoQuerySnapshot = await alunoQuery.GetSnapshotAsync();
                List<Aluno> listaAluno = new List<Aluno>();

                foreach (DocumentSnapshot documentSnapshot in alunoQuerySnapshot.Documents)
                {
                    if (documentSnapshot.Exists)
                    {
                        Dictionary<string, object> city = documentSnapshot.ToDictionary();
                        string json = JsonConvert.SerializeObject(city);
                        Aluno novoAluno = JsonConvert.DeserializeObject<Aluno>(json);
                        novoAluno.AlunoId = documentSnapshot.Id;
                        listaAluno.Add(novoAluno);
                    }
                }

                List<Aluno> listaAlunoOrdenada = listaAluno.OrderBy(x => x.Nome).ToList();
                return listaAlunoOrdenada;
            }
            catch(Exception ex)
            {
                var erro = ex.Message;
                throw;
            }
        }
        public async Task<Aluno> GetAluno(string id)
        {
            try
            {
                DocumentReference docRef = fireStoreDb.Collection("alunos").Document(id);
                DocumentSnapshot snapshot = await docRef.GetSnapshotAsync();

                if (snapshot.Exists)
                {
                    Aluno aluno = snapshot.ConvertTo<Aluno>();
                    aluno.AlunoId = snapshot.Id;
                    return aluno;
                }
                else
                {
                    return new Aluno();
                }
            }
            catch
            {
                throw;
            }
        }
        public async void AddAluno(Aluno aluno)
        {
            try
            {
                CollectionReference colRef = fireStoreDb.Collection("alunos");
                await colRef.AddAsync(aluno);
            }
            catch
            {
                throw;
            }
        }
        public async void UpdateAluno(Aluno aluno)
        {
            try
            {
                DocumentReference alunoRef = fireStoreDb.Collection("alunos").Document(aluno.AlunoId);
                await alunoRef.SetAsync(aluno, SetOptions.Overwrite);
            }
            catch
            {
                throw;
            }
        }
        public async void DeleteAluno(string id)
        {
            try
            {
                DocumentReference alunoRef = fireStoreDb.Collection("alunos").Document(id);
                await alunoRef.DeleteAsync();
            }
            catch
            {
                throw;
            }
        }
        public async Task<List<Cidade>> GetCidades()
        {
            try
            {
                Query cidadesQuery = fireStoreDb.Collection("cidades");
                QuerySnapshot cidadesQuerySnapshot = await cidadesQuery.GetSnapshotAsync();
                List<Cidade> listaCidades = new List<Cidade>();

                foreach (DocumentSnapshot documentSnapshot in cidadesQuerySnapshot.Documents)
                {
                    if (documentSnapshot.Exists)
                    {
                        Dictionary<string, object> city = documentSnapshot.ToDictionary();
                        string json = JsonConvert.SerializeObject(city);
                        Cidade novaCidade = JsonConvert.DeserializeObject<Cidade>(json);
                        listaCidades.Add(novaCidade);
                    }
                }
                return listaCidades;
            }
            catch
            {
                throw;
            }
        }     
    }
}
